{% extends "basic/layout.html" %}
{% set script_files = script_files + ['_static/bootstrap.js', '_static/bootstrap-sphinx.js'] %}
{% set css_files = ['_static/bootstrap.css', '_static/bootstrap-sphinx.css'] + css_files %}
{% set display_github = False %}

{# Sidebar: Rework into our Boostrap nav section. #}
{% macro navBar() %}
  <div id="navbar" class="navbar navbar-fixed-top">
    <div class="navbar-inner">
      <div class="container-fluid">
        <a class="brand" href="{{ pathto(master_doc) }}">{{ project|e }}</a>
        <span class="navbar-text pull-left"><b>{{ version|e }}</b></span>
          <ul class="nav">
            <li class="divider-vertical"></li>
            {% block sidebartoc %}
              {% include "globaltoc.html" %}
              {% include "localtoc.html" %}
            {% endblock %}
            {% block sidebarrel %}
              {% include "relations.html" %}
            {% endblock %}
          </ul>
          <ul class="nav pull-right">
            {% block sidebarsourcelink2 %}
              {% include "sourcelink.html" %}
            {% endblock %}
          {% block sidebarsearch %}
            {% include "searchbox.html" %}
          {% endblock %}
          </ul>
        </div>
      </div>
    </div>
  </div>
{% endmacro %}

{% block header %}{{ navBar() }}{% endblock %}

{# Silence the sidebar's, relbar's #}
{% block sidebar1 %}{% endblock %}
{% block sidebar2 %}{% endblock %}
{% block relbar1 %}{% endblock %}
{% block relbar2 %}{% endblock %}

{%- block content %}
<div class="container content">
   {% block body %} {% endblock %}
</div>
{%- endblock %}

{%- block footer %}
<hr>

<footer class="footer">
  <div class="container">
    <p class="pull-right"><a href="#">Back to top ↑</a></p>
    <ul class="unstyled muted">
    {%- if show_copyright %}
      <li><small>
      {%- if hasdoc('copyright') %}
        {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
      {%- else %}
        {% trans copyright=copyright|e %}&copy; {{ copyright }}.{% endtrans %}<br/>
      {%- endif %}
      </small></li>
    {%- endif %}
    {%- if last_updated %}
      <li><small>
      {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}<br/>
      </small></li>
    {%- endif %}
    {%- if show_sphinx %}
      <li><small>
      {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx.pocoo.org/">Sphinx</a> {{ sphinx_version }}.{% endtrans %}
      </small></li>
    {%- endif %}
    </ul>
  </div>
</footer>
{%- endblock %}

